local mainFrame = CreateFrame("frame", "SpeedometerMainFrame");
indFrame = CreateFrame("button", "SpeedometerIndFrame", mainFrame);
Speedo = {};

local MframeW, MframeH;
MframeW = 220;
MframeH = 220;

local Iframe, IframeH;
IframeW = 220;
IframeH = 220;

local currentRot = 0;
local pi = 3.14159265359;

local timer = 0;

Speedometer_showFrame = true;
Speedometer_curStyle = "DEFAULT_STYLE";
Speedometer_frameScale = 1;
Speedometer_framebufferSize = 10;
Speedometer_isLocked = false;
Speedometer_format = "KMH";

Speedometer = {};

Speedometer.styles = {"deathknight", "demonhunter", "druid", "hunter", "mage", "monk", "paladin", "priest", "rogue", "shaman", "warlock", "warrior" };

mainFrame:SetWidth(MframeW);
mainFrame:SetHeight(MframeH);
mainFrame:EnableMouse(true);
mainFrame:SetPoint("CENTER", 0, 0, UIParent);
mainFrame:SetMovable(true);
mainFrame:RegisterForDrag("LeftButton");
mainFrame:SetFrameStrata("Background");
mainFrame:SetUserPlaced(true);
mainFrame:SetScript("OnDragStart", mainFrame.StartMoving);
mainFrame:SetScript("OnDragStop", mainFrame.StopMovingOrSizing);

mainFrame:RegisterEvent("ADDON_LOADED");
mainFrame:RegisterEvent("CINEMATIC_START");
mainFrame:RegisterEvent("CINEMATIC_STOP");

indFrame:SetWidth(IframeW);
indFrame:SetHeight(IframeH);
indFrame:EnableMouse(false);
indFrame:SetPoint("CENTER", 0, 4, mainFrame);
indFrame:SetMovable(false);
indFrame:RegisterForDrag("LeftButton");
indFrame:SetFrameStrata("Background");
indFrame:SetEnabled(false);

local ntexA = indFrame:CreateTexture();

local function SetLocked()
	if(Speedometer_isLocked == true) then
		mainFrame:EnableMouse(false);
	else
		mainFrame:EnableMouse(true);
	end
end

local function round(num, numDecimalPlaces)
  local mult = 10^(numDecimalPlaces or 0);
  return math.floor(num * mult + 0.5) / mult;
end

local function InvertBool(b)
	if (b == true) then 
		return false;
	else 
		return true;
	end
end

local function PrintMsg(s)
	print("|cFF8514FFSpeedometer:|r " .. s);
end

local function LoadStyle(styleName)

	mainFrame:SetBackdrop({
      bgFile="Interface\\Addons\\Speedometer\\color_variants\\speedometer_" .. styleName,
      tile=1, tileSize=MframeW * Speedometer_frameScale, edgeSize=32, 
      insets={left=11, right=12, top=12, bottom=11};})
	  
	ntexA:SetTexture("Interface\\Addons\\Speedometer\\color_variants\\indicator_" .. styleName);
	ntexA:SetTexCoord(0, 1, 0, 1);
	ntexA:SetAllPoints()	;
	indFrame:SetDisabledTexture(ntexA);
	  
	Speedometer_curStyle = styleName;
end

Speedometer.SetRenderMainFrame = function(state)

	if (state == true) then
		mainFrame:Show();
		indFrame:Show();
		Speedometer_showFrame = true;
	else
		mainFrame:Hide();
		indFrame:Hide();
		Speedometer_showFrame = false;
	end
end

Speedometer.SetFrameScale = function(s)
	mainFrame:SetScale(s);
	indFrame:SetScale(s);
	Speedometer_frameScale = round(s, 3);
	
	LoadStyle(Speedometer_curStyle);
end

Speedometer.SetFramebufferSize = function(s)
	frameBuffer = {};
	Speedometer_framebufferSize = s;
	
	local count = 0;
	for i, v in pairs(frameBuffer) do
		count = count + 1;
	end
	
	for i = 1, s, 1 do
		frameBuffer[s] = 0;
	end
	
end

Speedometer.SetFramebufferSize(5);

local function NilToZero(i)
	if(i == nil) then return 0;
	else return i;
	end
end

local function YDStoKMH(yds)
	local vel = yds;
	vel = vel / 1.0936;
	vel = vel / 1000;
	vel = vel * 60 * 60;
	return vel;
end

local function YDStoMPH(yds)
	local vel = yds;
	vel = vel * 0.00056818;
	vel = vel * 60 * 60;
	return vel;
end

local function Clamp(min, max, val)
	if(val < min) then return min;
	elseif (val > max) then return max;
	else return val;
	end
end

local function Lerp(a, b, t)
	if(a >= 0) then
		local delta = b - a;
		delta = delta * t;
		return a + delta;
	
	elseif (a < 0) then
		local delta = a - b;
		delta = delta * t;
		return b + delta;
		
	end
end

local function SetRotationDegrees(d)
	local mx, my = GetCursorPosition();
	ntexA:SetRotation((-(d-90) * pi) / 180, 0.5, 0.31);
end

local function TranslateSpeed(vel)

	if (Speedometer_format == "KMH") then
		return YDStoKMH(vel);
	elseif (Speedometer_format == "MPH") then
		return YDStoMPH(vel);
	else
		return YDStoKMH(vel);
	end
end

local function UpdateHandler(deltaTime)
	timer = timer + deltaTime;

	local vel = GetUnitSpeed("player");
	local vel = TranslateSpeed(vel);
	local newRot = Clamp(0, 180, (vel / 160) * 180);
	
	local count = 0;
	local sum = 0;
	for i, v in pairs(frameBuffer) do
		count = count + 1;
	end
	for i, v in pairs(frameBuffer) do
		if(i > 1) then
			frameBuffer[i - 1] = frameBuffer[i];
		end
		
		if(i == count) then
			frameBuffer[i] = newRot;
		end
		
		sum = sum + v;
	end
	
	local avg = sum / count;
	
	
	SetRotationDegrees(avg);

end
indFrame:SetScript("OnUpdate", function(self, deltaTime) UpdateHandler(deltaTime); end);

local function eventHandler(self, event, ...)

	if(event == "ADDON_LOADED") then
		addonName = select(1, ...);
		if(addonName == "Speedometer") then
		
			if (Speedometer_curStyle == "DEFAULT_STYLE") then
				local classname = select(2, UnitClass("player"));
				classname = string.lower(classname);
				for i, v in pairs(Speedometer.styles) do
					if(classname == v) then LoadStyle(classname) end
				end
			end
		
			Speedometer.SetRenderMainFrame(Speedometer_showFrame);
			Speedometer.SetFramebufferSize(Speedometer_framebufferSize);
			LoadStyle(Speedometer_curStyle);
			SetLocked(Speedometer_isLocked);
			mainFrame:SetUserPlaced(true);
			
		end
		
	elseif (event == "CINEMATIC_START") then
		mainFrame:Hide();
		indFrame:Hide();
	
	elseif (event == "CINEMATIC_STOP") then
		if (Speedometer_showFrame == true) then
			mainFrame:Show();
			indFrame:Hide();
		end
	
	end
end
mainFrame:SetScript("OnEvent", eventHandler);

local function PrintHelp()
	PrintMsg("Available commands:");
	PrintMsg("/speedo open -> Opens the speedometer.");
	PrintMsg("/speedo close -> Closes the speedometer");
	PrintMsg("/speedo toggle -> Toggles the speedometer.");
	PrintMsg("/speedo -> Toggles the speedometer.");
	PrintMsg("/speedo pos -> Resets the speedometers position.");
	PrintMsg("/speedo lock -> Locks the speedometer.");
	PrintMsg("/speedo unlock -> Unlocks the speedometer.");
	PrintMsg("/speedo help -> Displays this message.");
	PrintMsg("/speedo set style <stylename> -> Switches to a style.");
	PrintMsg("/speedo set format <kmh / mph> -> Switches between KMH and MPH.");
	PrintMsg("/speedo set gravity <number> -> Sets how inert the speedometer is. The higher, the slower.");
	PrintMsg("/speedo list styles -> Lists all available styles.");
end

local function PrintStyles()
	PrintMsg("Available styles: ");

	for i, v in pairs(Speedometer.styles) do
			PrintMsg(v);
		end
		PrintMsg("currentClass");
end

SLASH_SPEEDO1 = "/speedometer";
SLASH_SPEEDO2 = "/speedo";
SLASH_SPEEDO3 = "/spd";
SlashCmdList["SPEEDO"] = function(msg)

	local comm, arg1, arg2 = strsplit(" ", msg);
	if(comm == nil) then comm = ""; end
	if(arg1 == nil) then arg1 = ""; end
	if(arg2 == nil) then arg2 = ""; end

	if (msg == "") then
		Speedometer.SetRenderMainFrame(InvertBool(Speedometer_showFrame));
	
	elseif (comm == "show") then
		Speedometer.SetRenderMainFrame(true);
	
	elseif (comm == "open") then
		Speedometer.SetRenderMainFrame(true);
		
	elseif (comm == "hide") then
		Speedometer.SetRenderMainFrame(false);
		
	elseif (comm == "close") then
		Speedometer.SetRenderMainFrame(false);
		
	elseif (comm == "toggle") then
		Speedometer.SetRenderMainFrame(InvertBool(Speedometer_showFrame));
		
	elseif (comm == "lock") then
		SetLocked(true);
		
	elseif (comm == "unlock") then
		SetLocked(false);
		
	elseif (comm == "pos") then
		mainFrame:SetUserPlaced(false);
		mainFrame:SetPoint("CENTER", 0, 0, UIParent);
		ReloadUI();
		
	elseif (comm == "list") then
		if (arg1 == "styles") then
			PrintStyles();
			
		else
			PrintHelp();
		end
	
	elseif (comm == "set") then
	
			if (arg1 == "style") then
				
				if (arg2 == "currentClass") then
					local classname = select(2, UnitClass("player"));
					classname = string.lower(classname);
					for i, v in pairs(Speedometer.styles) do
					if(classname == v) then LoadStyle(classname) end
					end

				elseif (arg2 ~= "") then
					doesExist = false;
					for i, v in pairs(Speedometer.styles) do
						if(arg2 == v) then doesExist = true; end
					end
					
					if (doesExist == true) then
						LoadStyle(arg2);
						PrintMsg("Set style to " .. arg2 .. "...");
					else
						PrintMsg("Style \"" .. arg2 .. "\" cannot be found...");
					end
					
				else
					PrintMsg("Which style? Available styles:");
					PrintStyles();
				end
			
			elseif (arg1 == "gravity") then
				if (arg2 ~= "") then
					local val = tonumber(arg2);
					
					if (val == nil) then
						PrintMsg("Invalid number!");
					else 
						if (val < 1) then 
							val = 1;
						elseif (val > 200) then
							val = 200;
						end
						
						Speedometer.SetFramebufferSize(val);
						PrintMsg("Set gravity to " .. val .. "...");
					end
					
					
				else
					PrintMsg("Please pass a value! Example:");
					PrintMsg("/speedo set gravity 10");
				end
	
			elseif (arg1 == "format") then
				if (string.lower(arg2) == "kmh") then
					Speedometer_format = "KMH";
					PrintMsg("Set format to kmh.");
				elseif (string.lower(arg2) == "mph") then
					Speedometer_format = "MPH";
					PrintMsg("Set format to mph.");
				else
					PrintMsg("Invalid format. Please use either KMH or MPH.");
				end
	
			else
				PrintHelp();
			end
		
	elseif (comm == "help") then
		PrintHelp();
		
	end
end
